@@ -88,7 +88,7 @@ from zim.signals import SignalEmitter, ConnectorMixin, SIGNAL_AFTER, SignalHandl
 from zim.actions import action, toggle_action, get_gtk_actiongroup
 from zim.utils import classproperty, get_module, lookup_subclass, WeakSet
 
-from zim.config import data_dirs, VirtualConfigManager
+from zim.config import data_dirs, VirtualConfigManager, XDG_DATA_HOME
 
 
 logger = logging.getLogger('zim.plugins')
@@ -114,6 +114,9 @@ __path__.append(__path__.pop(0)) # reshuffle real module path to the end
 #~ print "PLUGIN PATH:", __path__
 
 
+PLUGIN_FOLDER = XDG_DATA_HOME.subdir('zim/plugins')
+
+
 class PluginManager(ConnectorMixin, collections.Mapping):
 	'''Manager that maintains a set of active plugins
 
@@ -181,7 +184,7 @@ class PluginManager(ConnectorMixin, collections.Mapping):
 		# List "zim.plugins" sub modules based on __path__ because this
 		# parameter determines what folders will considered when importing
 		# sub-modules of the this package once this module is loaded.
-		plugins = set(['backlinkpane', 'zeitgeist-logger', 'linesorter', 'diagrameditor', 'arithmetic', 'quicknote', 'bookmarksbar', 'printtobrowser', 'spell', 'ditaaeditor', 'insertsymbol', 'screenshot', 'sequencediagrameditor', 'sourceview', 'tasklist', 'attachmentbrowser', 'scoreeditor', 'equationeditor', 'gnuplot_ploteditor', 'tags', 'calendar', 'inlinecalculator', 'distractionfree', 'trayicon', 'tableeditor', 'linkmap', 'versioncontrol', 'gnu_r_ploteditor', 'tableofcontents']) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY
+		plugins = set(['arithmetic', 'attachmentbrowser', 'backlinkpane', 'bookmarksbar', 'calendar', 'diagrameditor', 'distractionfree', 'ditaaeditor', 'equationeditor', 'gnu_r_ploteditor', 'gnuplot_ploteditor', 'inlinecalculator', 'insertsymbol', 'linesorter', 'linkmap', 'printtobrowser', 'quicknote', 'scoreeditor', 'screenshot', 'sequencediagrameditor', 'sourceview', 'spell', 'tableeditor', 'tableofcontents', 'tags', 'tasklist', 'trayicon', 'versioncontrol', 'zeitgeist-logger']) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY
 		for dir in __path__:
 			dir = Dir(dir)
 			for candidate in dir.list(): # returns [] if dir does not exist
@@ -378,9 +381,6 @@ class PluginClass(ConnectorMixin, SignalEmitter):
 	in the plugin module
 
 	@ivar extensions: a set with extension objects loaded by this plugin.
-	The lookup extensions objects it is usually better to use the methods
-	L{get_extension()} or L{get_extensions()} rather than using this
-	set directly.
 
 	@signal: C{extension-point-changed (name)}: emitted when extension
 	point C{name} changes
@@ -542,14 +542,15 @@ class PluginClass(ConnectorMixin, SignalEmitter):
 		@param extends: class name of the to-be-extended object
 		'''
 		klass = self.extension_classes.pop(extends)
-		for obj in self.get_extensions(klass):
-			obj.destroy()
+		for obj in self.extensions:
+			if isinstance(obj, klass):
+				obj.destroy()
 
 	def extend(self, obj, _name=None):
 		'''This method will look through the extensions defined for this
 		plugin and construct a new extension object if a match is found
 		for C{obj}.
-		@param obj: the obejct to be extended
+		@param obj: the object to be extended
 		@param _name: lookup name to use when extending the object.
 		To be used for testing only. Normally the class name of C{obj}
 		is used.
@@ -559,31 +560,32 @@ class PluginClass(ConnectorMixin, SignalEmitter):
 			ext = self.extension_classes[name](self, obj)
 			self.extensions.add(ext)
 
-	def get_extension(self, klass, **attr):
-		'''Look up an extension object instatiation
-		@param klass: the class of the extention object (_not_ the to-be-extended
-		klass)
-		@param attr: any object attributes that should match
-		@returns: a single extension object or C{None}
+	def get_extensions(self, obj):
+		'''Look up any extensions for C{obj} managed by this plugin
+		@param obj: the extended object
+		@returns: a list of extension objects
 		'''
-		ext = self.get_extensions(klass)
-		for key, value in attr.items():
-			ext = filter(lambda e: getattr(e, key) == value, ext)
-
-		if len(ext) > 1:
-			raise AssertionError, 'BUG: multiple extensions of class %s found' % klass
-		elif ext:
-			return ext[0]
+		if hasattr(obj, '__zim_extension_objects__'):
+			return [e for e in obj.__zim_extension_objects__ if e in self.extensions]
 		else:
-			return None
+			return []
 
-	def get_extensions(self, klass):
-		'''Look up extension object instatiations
+	def get_extension(self, obj, klass):
+		'''Look up an extension object instatiation
+		@param obj: the extended object
 		@param klass: the class of the extention object (_not_ the to-be-extended
 		klass)
-		@returns: a list of extension objects (if any)
+		@returns: a single extension object
+		@raises ValueError: if no extension was found
 		'''
-		return [e for e in self.extensions if isinstance(e, klass)]
+		exts = [e for e in self.get_extensions(obj) if isinstance(e, klass)]
+
+		if len(exts) == 1:
+			return exts[0]
+		elif len(exts) > 1:
+			raise AssertionError, 'BUG: multiple extensions found of class: %s' % klass
+		else:
+			raise ValueError, 'No extension of class: %s' % klass
 
 	def destroy(self):
 		'''Destroy the plugin object and all extensions
@@ -599,9 +601,16 @@ class PluginClass(ConnectorMixin, SignalEmitter):
 
 		try:
 			self.disconnect_all()
+			self.teardown()
 		except:
 			logger.exception('Exception while disconnecting %s', self)
 
+	def teardown(self):
+		'''Cleanup method called by C{destroy()}.
+		Can be implemented by sub-classes.
+		'''
+		pass
+
 
 def extends(eklass, autoload=True):
 	'''Class decorator to define extension classes
@@ -648,6 +657,8 @@ class ObjectExtension(SignalEmitter, ConnectorMixin):
 	@ivar obj: the object being extended
 	'''
 
+	__signals__ = {}
+
 	def __init__(self, plugin, obj):
 		'''Constructor
 		@param plugin: the plugin object to which this extension belongs
@@ -693,7 +704,7 @@ class ObjectExtension(SignalEmitter, ConnectorMixin):
 	def teardown(self):
 		'''Remove changes made by B{this} class from the extended object
 		To be overloaded by child classes
-		@note: do not call parent class C{remove()} here, that is
+		@note: do not call parent class C{teardown()} here, that is
 		already taken care of by C{destroy()}
 		'''
 		self.disconnect_all()
@@ -747,10 +758,9 @@ class WindowExtension(ObjectExtension):
 			self.uistate = None
 
 		if hasattr(self, 'uimanager_xml'):
-			# XXX TODO move uimanager to window
 			actiongroup = get_gtk_actiongroup(self)
-			self.window.ui.uimanager.insert_action_group(actiongroup, 0)
-			self._uimanager_id = self.window.ui.uimanager.add_ui_from_string(self.uimanager_xml)
+			self.window.uimanager.insert_action_group(actiongroup, 0)
+			self._uimanager_id = self.window.uimanager.add_ui_from_string(self.uimanager_xml)
 
 		self.connectto(window, 'destroy')
 
@@ -761,12 +771,12 @@ class WindowExtension(ObjectExtension):
 		# TODO move uimanager to window
 		if hasattr(self, '_uimanager_id') \
 		and self._uimanager_id is not None:
-			self.window.ui.uimanager.remove_ui(self._uimanager_id)
+			self.window.uimanager.remove_ui(self._uimanager_id)
 			self._uimanager_id = None
 
 		if hasattr(self, 'actiongroup') \
 		and self.actiongroup is not None:
-			self.window.ui.uimanager.remove_action_group(self.actiongroup)
+			self.window.uimanager.remove_action_group(self.actiongroup)
 
 
 class DialogExtension(WindowExtension):
